@@ -32,7 +32,9 @@ class H2OGLMClassifier(H2OGLM):
                  betaConstraints=None,
                  interactionPairs=None,
                  columnsToCategorical=[],
+                 keepBinaryModels=False,
                  withContributions=False,
+                 dataFrameSerializer="ai.h2o.sparkling.utils.JSONDataFrameSerializer",
                  withLeafNodeAssignments=False,
                  namedMojoOutputColumns=True,
                  convertInvalidNumbersToNa=False,
@@ -82,6 +84,7 @@ class H2OGLMClassifier(H2OGLM):
                  maxConfusionMatrixSize=20,
                  computePValues=False,
                  removeCollinearCols=False,
+                 generateScoringHistory=False,
                  modelId=None,
                  nfolds=0,
                  keepCrossValidationModels=True,
@@ -101,16 +104,14 @@ class H2OGLMClassifier(H2OGLM):
                  stoppingTolerance=0.001,
                  customMetricFunc=None,
                  exportCheckpointsDir=None,
-                 aucType="AUTO",
-                 **kwargs):
+                 aucType="AUTO"):
         Initializer.load_sparkling_jar()
         super(H2OGLM, self).__init__()
         self._java_obj = self._new_java_obj("ai.h2o.sparkling.ml.algos.classification.H2OGLMClassifier", self.uid)
         self._setDefaultValuesFromJava()
         kwargs = Utils.getInputKwargs(self)
-        if 'distribution' in kwargs:
-            del kwargs['distribution']
-            warn("The parameter 'distribution' is deprecated and will be removed in the version 3.34.")
+        kwargs = self._updateInitKwargs(kwargs)
+
         if 'interactionPairs' in kwargs:
             warn("Interaction pairs are not supported!")
         self._set(**kwargs)
