@@ -30,7 +30,9 @@ class H2OXGBoostClassifier(H2OXGBoost):
                  calibrationDataFrame=None,
                  ignoredCols=None,
                  columnsToCategorical=[],
+                 keepBinaryModels=False,
                  withContributions=False,
+                 dataFrameSerializer="ai.h2o.sparkling.utils.JSONDataFrameSerializer",
                  withLeafNodeAssignments=False,
                  namedMojoOutputColumns=True,
                  convertInvalidNumbersToNa=False,
