@@ -31,6 +31,7 @@ class H2ORuleFitRegressor(H2ORuleFit):
                  columnsToCategorical=[],
                  keepBinaryModels=False,
                  withContributions=False,
+                 dataFrameSerializer="ai.h2o.sparkling.utils.JSONDataFrameSerializer",
                  withLeafNodeAssignments=False,
                  namedMojoOutputColumns=True,
                  convertInvalidNumbersToNa=False,
@@ -52,7 +53,8 @@ class H2ORuleFitRegressor(H2ORuleFit):
                  modelId=None,
                  distribution="AUTO",
                  labelCol="label",
-                 weightCol=None):
+                 weightCol=None,
+                 aucType="AUTO"):
         Initializer.load_sparkling_jar()
         super(H2ORuleFit, self).__init__()
         self._java_obj = self._new_java_obj("ai.h2o.sparkling.ml.algos.regression.H2ORuleFitRegressor", self.uid)
