@@ -1,6 +1,4 @@
 # -*- encoding: utf-8 -*-
-from __future__ import absolute_import, division, print_function, unicode_literals
-
 import functools as ft
 from inspect import getdoc
 import re
@@ -11,7 +9,6 @@ from h2o.estimators import H2OEstimator
 from h2o.exceptions import H2OResponseError, H2OValueError
 from h2o.frame import H2OFrame
 from h2o.job import H2OJob
-from h2o.utils.compatibility import *  # NOQA
 from h2o.utils.shared_utils import check_id
 from h2o.utils.typechecks import assert_is_type, is_type, numeric
 from ._base import H2OAutoMLBaseMixin, _fetch_state
@@ -76,33 +73,22 @@ class H2OAutoML(H2OAutoMLBaseMixin, Keyed):
     Automatic Machine Learning
 
     The Automatic Machine Learning (AutoML) function automates the supervised machine learning model training process.
-    It trains several models, cross-validated by default, by using the following available algorithms:
-    
-    - XGBoost
-    - GBM (Gradient Boosting Machine)
-    - GLM (Generalized Linear Model)
-    - DRF (Distributed Random Forest)
-    - XRT (eXtremely Randomized Trees)
-    - DeepLearning (Fully Connected Deep Neural Network)
-    
-    It also applies HPO on the following algorithms:
-    
-    - XGBoost
-    - GBM
-    - DeepLearning
-    
+    The current version of AutoML trains and cross-validates the following algorithms (in the following order):
+    three pre-specified XGBoost GBM (Gradient Boosting Machine) models,
+    a fixed grid of GLMs,
+    a default Random Forest (DRF),
+    five pre-specified H2O GBMs,
+    a near-default Deep Neural Net,
+    an Extremely Randomized Forest (XRT),
+    a random grid of XGBoost GBMs,
+    a random grid of H2O GBMs,
+    and a random grid of Deep Neural Nets.
     In some cases, there will not be enough time to complete all the algorithms, so some may be missing from the
-    leaderboard. 
-    Finally, AutoML also trains several Stacked Ensemble models at various stages during the run.
-    Mainly two kinds of Stacked Ensemble models are trained:
-    
-    - one of all available models at time t
-    - one of only the best models of each kind at time t.
-    
+    leaderboard.  AutoML trains several Stacked Ensemble models during the run.
+    Two kinds of Stacked Ensemble models are trained one of all available models, and one of only the best models of each kind.
     Note that Stacked Ensemble models are trained only if there isn't another stacked ensemble with the same base models.
 
     :examples:
-    
     >>> import h2o
     >>> from h2o.automl import H2OAutoML
     >>> h2o.init()
@@ -127,14 +113,13 @@ class H2OAutoML(H2OAutoMLBaseMixin, Keyed):
     """
 
     def __init__(self,
-                 nfolds=-1,
+                 nfolds=5,
                  balance_classes=False,
                  class_sampling_factors=None,
                  max_after_balance_size=5.0,
                  max_runtime_secs=None,
                  max_runtime_secs_per_model=None,
                  max_models=None,
-                 distribution="AUTO",
                  stopping_metric="AUTO",
                  stopping_tolerance=None,
                  stopping_rounds=3,
@@ -158,7 +143,7 @@ class H2OAutoML(H2OAutoMLBaseMixin, Keyed):
         
         :param int nfolds: Number of folds for k-fold cross-validation.
             Use ``0`` to disable cross-validation; this will also disable Stacked Ensemble (thus decreasing the overall model performance).
-            Defaults to ``-1``.
+            Defaults to ``5``.
 
         :param bool balance_classes: Specify whether to oversample the minority classes to balance the class distribution. This option can increase
             the data frame size. This option is only applicable for classification. If the oversampled size of the dataset exceeds the maximum size
@@ -170,20 +155,12 @@ class H2OAutoML(H2OAutoMLBaseMixin, Keyed):
             Requires ``balance_classes``.
             Defaults to ``5.0``.
         :param int max_runtime_secs: Specify the maximum time that the AutoML process will run for.
-            If both ``max_runtime_secs`` and ``max_models`` are specified, then the AutoML run will stop as soon as it hits either of these limits.
-            If neither ``max_runtime_secs`` nor ``max_models`` are specified, then ``max_runtime_secs`` dynamically
+            If neither ``max_runtime_secs`` nor ``max_models`` are specified by the user, then ``max_runtime_secs`` dynamically
             defaults to 3600 seconds (1 hour). Otherwise, defaults to ``0`` (no limit).
         :param int max_runtime_secs_per_model: Controls the max time the AutoML run will dedicate to each individual model.
             Defaults to ``0`` (disabled: no time limit).
-            Note that models constrained by a time budget are not guaranteed reproducible.
         :param int max_models: Specify the maximum number of models to build in an AutoML run, excluding the Stacked Ensemble models.
             Defaults to ``None`` (disabled: no limitation).
-            Always set this parameter to ensure AutoML reproducibility: all models are then trained until convergence and none is constrained by a time budget.
-        :param Union[str, dict] distribution: Distribution function used by algorithms that support it; other algorithms
-            use their defaults.  Possible values: "AUTO", "bernoulli", "multinomial", "gaussian", "poisson", "gamma",
-            "tweedie", "laplace", "quantile", "huber", "custom", and for parameterized distributions dictionary form is
-            used to specify the parameter, e.g., ``dict(type="tweedie", tweedie_power=1.5)``.
-            Defaults to ``AUTO``.
         :param str stopping_metric: Specifies the metric to use for early stopping. 
             The available options are:
             ``"AUTO"`` (This defaults to ``"logloss"`` for classification, ``"deviance"`` for regression),
@@ -208,27 +185,19 @@ class H2OAutoML(H2OAutoMLBaseMixin, Keyed):
             More models can be trained on an existing AutoML project by specifying the same project name in multiple calls to the AutoML function
             (as long as the same training frame, or a sample, is used in subsequent runs).
         :param exclude_algos: List the algorithms to skip during the model-building phase. 
-            The full list of options is:
-            
-                - ``"DRF"`` (Random Forest and Extremely-Randomized Trees)
-                - ``"GLM"``
-                - ``"XGBoost"``
-                - ``"GBM"``
-                - ``"DeepLearning"``
-                - ``"StackedEnsemble"``
-                
+            The full list of options is: 
+                ``"DRF"`` (Random Forest and Extremely-Randomized Trees),
+                ``"GLM"``,
+                ``"XGBoost"``,
+                ``"GBM"``,
+                ``"DeepLearning"``,
+                ``"StackedEnsemble"``. 
             Defaults to ``None``, which means that all appropriate H2O algorithms will be used, if the search stopping criteria allow. Optional.
-            Usage example::
-            
-                exclude_algos = ["GLM", "DeepLearning", "DRF"]
-                
+            Usage example: ``exclude_algos = ["GLM", "DeepLearning", "DRF"]``.
         :param include_algos: List the algorithms to restrict to during the model-building phase.
             This can't be used in combination with ``exclude_algos`` param.
             Defaults to ``None``, which means that all appropriate H2O algorithms will be used, if the search stopping criteria allow. Optional.
-            Usage example::
-
-                include_algos = ["GLM", "DeepLearning", "DRF"]
-                
+            Usage example: ``include_algos = ["GLM, "DeepLearning", "DRF"]``.
         :param exploitation_ratio: The budget ratio (between 0 and 1) dedicated to the exploitation (vs exploration) phase.
             By default, the exploitation phase is ``0`` (disabled) as this is still experimental;
             to activate it, it is recommended to try a ratio around 0.1.
@@ -255,7 +224,7 @@ class H2OAutoML(H2OAutoMLBaseMixin, Keyed):
             Defaults to ``"AUTO"`` (This translates to ``"auc"`` for binomial classification, ``"mean_per_class_error"`` for multinomial classification, ``"deviance"`` for regression).
         :param export_checkpoints_dir: Path to a directory where every model will be stored in binary form.
         :param verbosity: Verbosity of the backend messages printed during training.
-            Available options are ``None`` (live log disabled), ``"debug"``, ``"info"``, ``"warn"`` or ``"error"``.
+            Available options are ``None`` (live log disabled), ``"debug"``, ``"info"`` or ``"warn"``.
             Defaults to ``"warn"``.
         """
 
@@ -296,7 +265,6 @@ class H2OAutoML(H2OAutoMLBaseMixin, Keyed):
 
         self.project_name = project_name
         self.nfolds = nfolds
-        self.distribution = distribution
         self.balance_classes = balance_classes
         self.class_sampling_factors = class_sampling_factors
         self.max_after_balance_size = max_after_balance_size
@@ -336,16 +304,14 @@ class H2OAutoML(H2OAutoMLBaseMixin, Keyed):
         return project_name
 
     def __validate_nfolds(self, nfolds):
-        assert nfolds in (-1, 0) or nfolds > 1, ("nfolds set to %s; use nfolds >=2 if you want cross-validated metrics "
-                                                 "and Stacked Ensembles or use nfolds = 0 to disable or nfolds = -1 to "
-                                                 "let h2o choose automatically." % nfolds)
+        assert nfolds == 0 or nfolds > 1, "nfolds set to %s; use nfolds >=2 if you want cross-validated metrics and Stacked Ensembles or use nfolds = 0 to disable." % nfolds
         return nfolds
 
     def __validate_modeling_plan(self, modeling_plan):
         if modeling_plan is None:
             return None
 
-        supported_aliases = PList(['all', 'defaults', 'grids'])
+        supported_aliases = ['all', 'defaults', 'grids']
 
         def assert_is_step_def(sd):
             assert 'name' in sd, "each definition must have a 'name' key"
@@ -414,45 +380,11 @@ class H2OAutoML(H2OAutoMLBaseMixin, Keyed):
     def __validate_frame(self, fr, name=None, required=False):
         return H2OFrame._validate(fr, name, required=required)
 
-    def __validate_distribution(self, distribution):
-        if is_type(distribution, str):
-            distribution = distribution.lower()
-            if distribution == "custom":
-                raise H2OValueError('Distribution "custom" has to be specified as a '
-                                    'dictionary with their respective parameters, e.g., '
-                                    '`dict(type = \"custom\", custom_distribution_func = \"...\"))`.')
-            return distribution
-        if is_type(distribution, dict):
-            dist = distribution["type"].lower()
-            allowed_distribution_parameters = dict(
-                custom='custom_distribution_func',
-                huber='huber_alpha',
-                quantile='quantile_alpha',
-                tweedie='tweedie_power'
-            )
-            assert distribution.get(allowed_distribution_parameters.get(dist)) is not None or len(distribution) == 1, (
-                    "Distribution dictionary should contain distribution and a distribution "
-                    "parameter. For example `dict(type=\"{}\", {}=...)`."
-            ).format(dist, allowed_distribution_parameters[dist])
-            if distribution["type"] == "custom" and "custom_distribution_func" not in distribution.keys():
-                raise H2OValueError('Distribution "custom" has to be specified as a '
-                            'dictionary with their respective parameters, e.g., '
-                            '`dict(type = \"custom\", custom_distribution_func = \"...\"))`.')
-            if allowed_distribution_parameters.get(dist) in distribution.keys():
-                setattr(self, "_"+allowed_distribution_parameters[dist], distribution[allowed_distribution_parameters[dist]])
-            return dist
-
-
     _extract_params_doc(getdoc(__init__))
     project_name = _aml_property('build_control.project_name', types=(None, str), freezable=True,
                                  validate_fn=__validate_project_name)
     nfolds = _aml_property('build_control.nfolds', types=(int,), freezable=True,
                            validate_fn=__validate_nfolds)
-    distribution = _aml_property('build_control.distribution', types=(str, dict), freezable=True, validate_fn=__validate_distribution)
-    _custom_distribution_func = _aml_property('build_control.custom_distribution_func', types=(str,), freezable=True)
-    _huber_alpha = _aml_property('build_control.huber_alpha', types=(numeric,), freezable=True)
-    _tweedie_power = _aml_property('build_control.tweedie_power', types=(numeric,), freezable=True)
-    _quantile_alpha = _aml_property('build_control.quantile_alpha', types=(numeric,), freezable=True)
     balance_classes = _aml_property('build_control.balance_classes', types=(bool,), freezable=True)
     class_sampling_factors = _aml_property('build_control.class_sampling_factors', types=(None, [numeric]), freezable=True)
     max_after_balance_size = _aml_property('build_control.max_after_balance_size', types=(None, numeric), freezable=True)
@@ -565,7 +497,6 @@ class H2OAutoML(H2OAutoMLBaseMixin, Keyed):
         :returns: An H2OAutoML object.
 
         :examples:
-        
         >>> # Set up an H2OAutoML object
         >>> aml = H2OAutoML(max_runtime_secs=30)
         >>> # Launch an AutoML run
@@ -686,20 +617,24 @@ class H2OAutoML(H2OAutoMLBaseMixin, Keyed):
         """
         the callback function used to print verbose info when polling AutoML job.
         """
-        levels = ['debug', 'info', 'warn', 'error']
-        if verbosity is None or verbosity.lower() not in levels:
+        levels = ['Debug', 'Info', 'Warn']
+        if verbosity is None or verbosity.capitalize() not in levels:
             return
+
+        levels = levels[levels.index(verbosity.capitalize()):]
         try:
             if job.progress > state.get('last_job_progress', 0):
-                events_table = _fetch_state(job.dest_key, properties=[], verbosity=verbosity)['json']['event_log_table']
+                # print("\nbar_progress={}, job_progress={}".format(bar_progress, job.progress))
+                events = _fetch_state(job.dest_key, properties=['event_log'])['event_log']
+                events = events[events['level'].isin(levels), :]
                 last_nrows = state.get('last_events_nrows', 0)
-                if len(events_table.cell_values) > last_nrows:
-                    events = zip(*events_table[last_nrows:][['timestamp', 'message']])
+                if events.nrows > last_nrows:
+                    fr = events[last_nrows:, ['timestamp', 'message']].as_data_frame(use_pandas=False, header=False)
                     print('')
-                    for r in events:
+                    for r in fr:
                         print("{}: {}".format(r[0], r[1]))
                     print('')
-                    state['last_events_nrows'] = len(events_table.cell_values)
+                    state['last_events_nrows'] = events.nrows
             state['last_job_progress'] = job.progress
         except Exception as e:
             print("Failed polling AutoML progress log: {}".format(e))
