@@ -8,10 +8,14 @@ This module provides helper functions to write code that is backward-compatible.
 # Note: no unicode_literals feature, since type.__getattribute__ cannot take unicode strings as parameter...
 from __future__ import division, print_function, absolute_import
 from h2o.utils.compatibility import *  # NOQA
+from h2o.utils.typechecks import _str_type
 
 from functools import wraps
+import inspect
 import warnings
 
+from h2o.exceptions import H2ODeprecationWarning
+
 
 def h2o_meta(*args):
     return with_metaclass(H2OMeta, *args)
@@ -31,9 +35,114 @@ def extend_and_replace(cls, **attrs):
     return new_cls
 
 
-class Deprecated(object):
+def decoration_info(fn):
+    return getattr(fn, '__decoration__', None)
+
+
+def _set_decoration_info(wrapper, wrapped, decoration_type):
+    wrapper.__decoration__ = dict(
+        wrapped=wrapped,
+        type=decoration_type
+    )
+    
+
+def deprecated_params(deprecations):
+    old = deprecations.keys()
+    
+    def decorator(fn):
+        fn_name = fullname(fn)
+        
+        @wraps(fn)
+        def wrapper(*args, **kwargs):
+            new_kwargs = {}
+            keys = set(kwargs.keys())
+            messages = []
+            for k, v in kwargs.items():
+                if k in old:
+                    new = deprecations[k]
+                    new_tup = (((lambda ov: None), None) if new in [None, ()] 
+                               else ((lambda ov: {new: ov}), None) if isinstance(new, _str_type)
+                               else (new, None) if callable(new)
+                               else ((lambda ov: None), new[1]) if isinstance(new, tuple) and new[0] is None
+                               else ((lambda ov: {new[0]: ov}), new[1]) if isinstance(new, tuple) and isinstance(new[0], _str_type)
+                               else new)
+                    assert isinstance(new_tup, tuple), (
+                            "`deprecations` values must be one of: "
+                            "None (deprecated param removed), a string (deprecated property renamed), "
+                            "a tuple(new_name: Optional[str], message: str) to customize the deprecation message, "
+                            "a callable lambda old_value: dict(param1=value1, param2=value2) for advanced deprecations "
+                            "(one param replaced with one or more params with transformation of the deprecated value), "
+                            "or a tuple(lambda old_value: dict(param1=value1, param2=value2), message: str).")
+                    transform_fn, msg = new_tup
+                    new_params = transform_fn(v)
+                    if new_params in [None, {}]:
+                        messages.append(msg or "``{}`` param of ``{}`` is deprecated and will be ignored."
+                                               .format(k, fn_name))
+                    else:
+                        assert isinstance(new_params, dict)
+                        messages.append(msg or "``{}`` param of ``{}`` is deprecated, please use ``{}`` instead."
+                                               .format(k, fn_name, ', '.join(new_params.keys())))
+                        intersect = set(new_params.keys()) & keys
+                        if any(intersect):
+                            messages.append("Using both deprecated param ``{}`` and new param(s) ``{}`` in call to ``{}``, "
+                                            "the deprecated param will be ignored."
+                                            .format(k, ', '.join(intersect), fn_name))
+                        else:
+                            new_kwargs.update(new_params)
+                else:
+                    new_kwargs[k] = v
+            for msg in messages:
+                warnings.warn(msg, H2ODeprecationWarning, 2)
+            return fn(*args, **new_kwargs)
+    
+        _set_decoration_info(wrapper, fn, 'deprecation')
+    
+        return wrapper
+    
+    return decorator
+        
+
+def deprecated_property(name, replaced_by=None, message=None):
     """
-    Decorator for deprecated methods.
+    Creates a deprecated property that forwards logic to `replaced_by` property.
+    :param name: name of the deprecated property.
+    :param replaced_by: the new property object. If None, then the deprecated property will be a no-op property.
+    :param message: the custom deprecation message. If None, a default message will be used.
+    :return: the deprecated property.
+    """
+    
+    if replaced_by:
+        new_name = replaced_by.fget.__name__
+        doc = message or "[Deprecated] Use ``{}`` instead".format(new_name)
+        # doc += "\n\n{}".format(replaced_by.__doc__)
+        msg = message or "``{}`` is deprecated, please use ``{}`` instead.".format(name, new_name)
+        
+        def wrap(accessor):
+            if accessor is None: return 
+            
+            def wrapper(*args):
+                warnings.warn(msg, H2ODeprecationWarning, 2)
+                return accessor(*args)
+            return wrapper
+        
+        return property(wrap(replaced_by.fget), wrap(replaced_by.fset), wrap(replaced_by.fdel), doc)
+    else:
+        doc = message or "[Deprecated] The property was removed and will be ignored."
+        msg = message or "``{}`` is deprecated and will be ignored.".format(name)
+        
+        def _fget(self):
+            warnings.warn(msg, H2ODeprecationWarning, 2)
+            return None
+            
+        def _fset(self, _):
+            warnings.warn(msg, H2ODeprecationWarning, 2)
+
+        return property(_fget, _fset, None, doc)
+
+
+class _DeprecatedFunction(object):
+    """
+    Decorator for deprecated functions or methods.
 
     :example::
         class Foo:
@@ -42,7 +151,7 @@ class Deprecated(object):
                 ...
                 do_sth(param)
 
-            @Deprecated(replaced_by=new_method)
+            @deprecated_function(replaced_by=new_method)
             def old_method(self, param=None):
                 pass
     """
@@ -58,20 +167,25 @@ class Deprecated(object):
         self._replaced_by = replaced_by
 
     def __call__(self, fn):
-        msg = self._msg if self._msg is not None \
-            else "{} is deprecated, please use ``{}`` instead.".format(fullname(fn), fullname(self._replaced_by)) if self._replaced_by is not None \
-            else "{} is deprecated.".format(fullname(fn))
+        msg = (self._msg if self._msg is not None
+               else "``{}`` is deprecated, please use ``{}`` instead."
+                    .format(fullname(fn), fullname(self._replaced_by)) 
+                    if self._replaced_by is not None
+               else "``{}`` is deprecated.".format(fullname(fn)))
         fn.__doc__ = "{msg}\n\n{doc}".format(msg=msg, doc=fn.__doc__) if fn.__doc__ is not None else msg
         call_fn = self._replaced_by or fn
 
         @wraps(fn)
         def wrapper(*args, **kwargs):
-            warnings.warn(msg, DeprecationWarning, 2)
+            warnings.warn(msg, H2ODeprecationWarning, 2)
             return call_fn(*args, **kwargs)
 
         return wrapper
 
 
+deprecated_fn = _DeprecatedFunction
+
+
 class MetaFeature(object):
     """To be implemented by meta features exposed through the ``H2OMeta` metaclass"""
 
@@ -107,15 +221,14 @@ class MetaFeature(object):
             return None
 
 
-
-class Alias(MetaFeature):
+class _Alias(MetaFeature):
     """
     Decorator to alias the current method without having to implement any duplicating or forwarding code.
 
    :example::
     class Foo(metaclass=H2OMeta):
 
-        @Alias('ein', 'uno')
+        @alias('ein', 'uno')
         def one(self, param):
             ...
             do_sth()
@@ -142,14 +255,17 @@ class Alias(MetaFeature):
     def __call__(self, fn):
         fn._aliases = self._aliases
         return fn
+    
+    
+alias = _Alias
 
 
-class BackwardsCompatible(MetaFeature):
+class _BackwardsCompatible(MetaFeature):
     """
     Decorator to keep backward compatibility support for old methods without exposing them (non-discoverable, non-documented).
 
     :example:
-        @BackwardsCompatible(
+        @backwards_compatibility(
             class_attrs=dict(
               counter=1
             ),
@@ -177,6 +293,7 @@ class BackwardsCompatible(MetaFeature):
         clz._bc = self
         new_clz = None
 
+        @wraps(clz.__init__)
         def __init__(self, *args, **kwargs):
             super(new_clz, self).__init__(*args, **kwargs)
             self._bci = {name: val.__get__(self, new_clz) if callable(val) else val for name, val in clz._bc._instance_attrs.items()}
@@ -199,7 +316,7 @@ class BackwardsCompatible(MetaFeature):
         bc = cls.type_attr(clz, '_bc')
         if bc is not None and name in bc._class_attrs:
             return bc._class_attrs[name]
-        return super(BackwardsCompatible, cls).get_class_attr(clz, name)
+        return super(_BackwardsCompatible, cls).get_class_attr(clz, name)
 
     @classmethod
     def set_class_attr(cls, clz, name, value):
@@ -207,7 +324,10 @@ class BackwardsCompatible(MetaFeature):
         if bc is not None and name in bc._class_attrs:
             bc._class_attrs[name] = value
             return True
-        return super(BackwardsCompatible, cls).set_class_attr(clz, name, value)
+        return super(_BackwardsCompatible, cls).set_class_attr(clz, name, value)
+
+
+backwards_compatibility = _BackwardsCompatible
 
 
 class H2OMeta(type):
@@ -216,7 +336,7 @@ class H2OMeta(type):
     Features requiring usage of this metaclass are listed and injected through the `_FEATURES` static field.
     """
 
-    _FEATURES = [Alias, BackwardsCompatible]
+    _FEATURES = [_Alias, _BackwardsCompatible]
 
     def __new__(mcs, name, bases, dct):
         for m in H2OMeta._FEATURES:
